%%the cell_data structure can be downloaded from the publication at https://www.embopress.org/doi/full/10.15252/msb.20178140
%%all the data is contained in what is named cell_data in the following scripts

clear all

%%Figure 6(c)i
%Simulated OCT4 expression including differentiating with an Allee effect

load cell_data %%data set 
load cckernel %%use cckernel from initial conditions script or other cell cycle kernel of your choice
load initialoctkernel %%use initialoct kernel from initial conditions script of kernel of your choice

ccpd=cckernel;

N=16; %starting number of cells
events=1000; %how many divisions do we want to occur
OCT4=zeros(10,100); %set up empty OCT4 matrix
r=0.025; %regulation parameter
k=1300; %carrying capacity
sigmaa=35; %additive noise
sigmam=0.035; %multiplicative noise
dt=1; %timestep
Atime=40; %Allee effect switch on 
a=1000; %Allee effect critical point
H=0.38; %Hurst exponent 

for i=1:N %for initial cells
cctime(i)=random(ccpd)*12; %cell initial cell cycle timeframes
cellstartingtime(i)= unifrnd(0,cctime(i)); %choose a random time between 0 and the cell cycle time for the cell to begin at 
cctimeframes(i)=cctime(i)-cellstartingtime(i); %remaining cell cycle
initialoct(i)=log(random(initialoctkernel)); %initial OCT4 
OCT4(1:floor(cctimeframes(i)),i)=loglogistic_bothnoise(initialoct(i), 1, floor(cctimeframes(i)), dt, k, r, sigmaa, sigmam,H); %%first cells OCT4 values
celldivisiontimes(i)=floor(cctimeframes(i)); %cell division time
if celldivisiontimes(i)==0 %correct for if the division time is 0 and make it 1 
    celldivisiontimes(i)=1;
end
end


for j=1:events %for every event
dividingcellID=find(celldivisiontimes==min(celldivisiontimes)); %find cell next dividing
if length(dividingcellID)>1 %if two cells want to divide at once pick one randomly
pos = randi(length(dividingcellID));
cellid=dividingcellID(pos);
else
    cellid=dividingcellID;
end

m=OCT4(:,cellid);
m=m(m~=0);
l=length(find(m<1));
if l==0 %%if the cell has no negative oct4 values then let it divide

%%if we pass allee switch on time 
    if celldivisiontimes(cellid)>Atime*12; %switch on allee effect
        ra=0.035; %regulation parameter in Allee effect model    
        if celldivisiontimes(cellid)-cctimes(cellid)<Atime*12; %%is the change happening in a cell's lifetime
            lastoctval=OCT4(Atime*12,cellid); %if so need to replace some of the values
            timelefttodiv=celldivisiontimes(cellid)-Atime*12; %time pre Allee effect 
            OCT4(Atime*12:Atime*12+timelefttodiv,i)=loglogistic_bothnoise_allee(lastoctval, Atime*12, Atime*12+timelefttodiv, dt, k, ra, sigmaa, sigmam, H, a); %replaced with allee effect time series  
        end
    
    newcctime1=random(ccpd)*12; %new cell cycle times
    newcctime2=random(ccpd)*12; 

    motherfinaloct=OCT4(celldivisiontimes(cellid),cellid); %note mother cell final OCT4 value
    
    %use loglogistic equation with Allee effect
    OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime1),cellid)= loglogistic_bothnoise_allee(motherfinaloct, 1, floor(newcctime1), dt, k, ra, sigmaa, sigmam, H, a); %update current cells
    OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime2),N+1)=loglogistic_bothnoise_allee(motherfinaloct, 1, floor(newcctime2), dt, k, ra, sigmaa, sigmam, H, a); %new cell
    celldivisiontimes(N+1)=celldivisiontimes(cellid)+floor(newcctime2); %new cell division time
    celldivisiontimes(cellid)= celldivisiontimes(cellid)+floor(newcctime1); %update cell division time for the cell that's dividing
    cctimes(cellid)=floor(newcctime1); %new cell cycle times
    cctimes(N+1)=floor(newcctime2);
    parentid(N+1)=cellid; %tracking parent IDs
    parentid(N)=cellid;
    N=N+1; %number of cells counter

    else %not past Allee switch on time, use loglogistic equation without Allee effect

    newcctime1=random(ccpd)*12; %new cell cycle times
    newcctime2=random(ccpd)*12; 

    motherfinaloct=OCT4(celldivisiontimes(cellid),cellid); %note mother cell final OCT4 value
    OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime1),cellid)= loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime1), dt, k, r, sigmaa, sigmam, H); %update current cells
    OCT4(celldivisiontimes(cellid)+1:celldivisiontimes(cellid)+floor(newcctime2),N+1)=loglogistic_bothnoise(motherfinaloct, 1, floor(newcctime2), dt, k, r, sigmaa, sigmam, H); %new cell
    celldivisiontimes(N+1)=celldivisiontimes(cellid)+floor(newcctime2); %new cell division time
    celldivisiontimes(cellid)= celldivisiontimes(cellid)+floor(newcctime1); %update cell division time for the cell that's dividing
    parentid(N+1)=cellid; %tracking parent IDs
    parentid(N)=cellid;
    cctimes(cellid)=floor(newcctime1); %cell cycle times
    cctimes(N+1)=floor(newcctime2);
    N=N+1; %number of cells counter
    end
    
end
end


%%chop OCTvals at 70 hours
OCT470h=OCT4(1:12*68,:);
OCT470h( :, ~any(OCT470h,1) ) = [];  %columns
timesteps=[1:1:length(OCT4)];

countpp=0;
OCT4pp=[];
for i=1:size(OCT470h,2)
    octvals=(OCT470h(:,i));
    octvals2=octvals(octvals~=0);
    octvals2=octvals2(~isnan(octvals2));
    if length(octvals2)>1
    starttime=find(octvals==octvals2(1)); 
    f=find(octvals2<.1);
    if length(f)>0
    octvals3=octvals2(1:f(1)-1);
    else 
    octvals3=octvals2;
    end
    if sum(diff(octvals3)>5)==0
        if octvals3(end)>log(500)
        OCT4pp(timesteps(starttime:starttime+length(octvals3)-1),countpp+1)=octvals3;
        cellidpp(countpp+1)=i;
        countpp=countpp+1;
        end 
    end 
    end 
end
countdiff=0;
OCT4diff=[];
for i=1:size(OCT470h,2)
    octvals=(OCT470h(:,i));
    octvals2=octvals(octvals~=0);
    octvals2=octvals2(~isnan(octvals2));
    if length(octvals2)>1
    starttime=find(octvals==octvals2(1)); 
    f=find(octvals2<.1);
        if length(f)>0
        octvals3=octvals2(1:f(1)-1);
        else 
        octvals3=octvals2;
        end
        if sum(diff(octvals3)>5)==0
             if octvals3(end)<log(500)
             OCT4diff(timesteps(starttime:starttime+length(octvals3)-1),countdiff+1)=octvals3;
             celliddiff(countdiff+1)=i;
             countdiff=countdiff+1;
             end
        end 
    end 
end


figure %plot the oct4 over time
for i=1:countpp
    oct=OCT4pp(:,i);
    f=find(oct~=0);
    plotoct=exp(oct(f));
    plot(timesteps(f)/12,plotoct,'Color',[0.3 0.3 0.8])
    hold on 
end 
for i=1:countdiff
    oct=OCT4diff(:,i);
    f=find(oct~=0);
    plotoct=exp(oct(f));
    plot(timesteps(f)/12,plotoct,'Color',[0.1 0.8 0.2])
    hold on 
end 
xlabel('Time (hours)')
ylabel('OCT4')
ax = gca;
ax.YAxis.Exponent = 3;
ylim([0 3100])
xlim([0 68])





%%Figure 6(c)ii
%mean OCT4 over time 
%set up groups for binning
groups=[1:(68*12-1)/70:68*12];
groups=round(groups);

%for pluripotent cells
for i=1:length(groups)-1
    selectoct=OCT4pp(groups(i):groups(i+1),:);
    f=find((selectoct>0) & (selectoct<10));
    selectoct=selectoct(f);
    expvals=exp(selectoct(:));
    expvals=expvals(expvals~=inf);
    meanpp(i)=mean(expvals);
    stdpp(i)=std(expvals);  
    medoctpp(i)=median(expvals);
    uqoctpp(i)=quantile(expvals,0.75);
    lqoctpp(i)=quantile(expvals,0.25);
    sdoctpp(i)=std(expvals);
    pdf1=fitdist(expvals,'Kernel','Bandwidth',50);
    xvals=[min(expvals):1:max(expvals)];
    yvals=pdf(pdf1,xvals);
    modeoctpp(i)=xvals(find(yvals==max(yvals)));
    selectoct=[];
end
groupcenterspp=(groups+circshift(groups,-1))/2;
groupcenterspp(end)=[];

%for differentiated cells
for i=1:length(groups)-1
    selectoct=OCT4diff(groups(i):groups(i+1),:);
    f=find((selectoct>0) & (selectoct<10));
    selectoct=selectoct(f);
    expvals=exp(selectoct(:));
    expvals=expvals(expvals~=inf);
    meandiff(i)=mean(expvals);
    stddiff(i)=std(expvals);
    medoctdiff(i)=median(expvals);
    uqoctdiff(i)=quantile(expvals,0.75);
    lqoctdiff(i)=quantile(expvals,0.25);
    sdoctdiff(i)=std(expvals);
    pdf1=fitdist(expvals,'Kernel','Bandwidth',200);
    xvals=[min(expvals):1:max(expvals)];
    yvals=pdf(pdf1,xvals);
    modeoctdiff(i)=xvals(find(yvals==max(yvals)));
    selectoct=[];
end
groupcentersdiff=(groups+circshift(groups,-1))/2;
groupcentersdiff(end)=[];

%plot figure (could do medians, uq and lq)
figure
shadedErrorBar(groupcenterspp/12,meanpp,stdpp,'lineprops',{'s-','LineStyle','none','Color',[0.3 0.3 0.8],'MarkerSize',4,'MarkerEdgeColor',[0.3 0.3 0.8],'MarkerFaceColor',[0.3 0.3 0.8]})
hold on
shadedErrorBar(groupcentersdiff/12,meandiff, stddiff,'lineprops',{'d-','LineStyle','none','Color',[0.1 0.8 0.2],'MarkerSize',3,'MarkerEdgeColor',[0.1 0.8 0.2],'MarkerFaceColor',[1 1 1]})
xlabel('Time (hours)')
ylabel('Mean OCT4')
xlim([0 68])
ylim([0 2100])
box on
ax = gca;
ax.YAxis.Exponent = 3;
yticks([0 1000 2000])

